package at.jku.ssw.stack.test;

import static org.junit.Assert.assertEquals;

import java.util.EmptyStackException;
import java.util.NoSuchElementException;

import org.junit.Assert;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.ExpectedException;

import at.jku.ssw.Iterator;
import at.jku.ssw.stack.Stack;

public abstract class StackTest {

	protected abstract Stack createStack();

	protected Stack createStack(Object... objects) {
		Stack stack = createStack();
		pushValues(stack, objects);
		return stack;
	}

	protected void pushValues(Stack stack, Object... objects) {
		for (Object o : objects) {
			stack.push(o);
		}
	}

	protected void assertContains(Stack stack, Object... objects) {
		Iterator it = stack.iterator();
		int i = 0;
		while (it.hasNext()) {
			Assert.assertEquals(objects[i++], it.next());
		}
		Assert.assertEquals(objects.length, i);
	}

	@Rule
	public ExpectedException thrown = ExpectedException.none();

	@Test
	public void testPush() {
		Stack stack = createStack(1, 5, 7);
		assertContains(stack, 7, 5, 1);
	}

	@Test
	public void testPop() {
		Stack stack = createStack(1, 5, 7);
		assertEquals(7, stack.pop());
		assertEquals(5, stack.pop());
		assertEquals(1, stack.pop());

		assertEquals(0, stack.size());
		thrown.expect(EmptyStackException.class);
		stack.pop();
	}

	@Test
	public void testPeek() {
		Stack stack = createStack(1, 5, 7);
		assertEquals(7, stack.peek());
		assertEquals(7, stack.peek());
		assertEquals(3, stack.size());
	}

	@Test
	public void testPeekEmpty() {
		Stack stack = createStack();
		thrown.expect(EmptyStackException.class);
		stack.peek();
	}

	@Test
	public void testPushDown() {
		Stack stack = createStack(1, 5, 7);
		stack.pushDown();
		assertContains(stack, 5, 1, 7);
	}

	@Test
	public void testPushDownEmpty() {
		Stack stack = createStack();
		thrown.expect(NoSuchElementException.class);
		stack.pushDown();
	}

	@Test
	public void testPushDownSingle() {
		Stack stack = createStack(1);
		stack.pushDown();
		assertContains(stack, 1);
	}

	@Test
	public void testReverse() {
		Stack stack = createStack(1, 5, 7);
		stack.reverse();
		assertContains(stack, 1, 5, 7);
	}

}
